import java.io.*;
import java.util.Arrays;
import java.util.Locale;
import java.util.StringTokenizer;

public class sol_ma_wa2 {
    private final InputReader reader;
    private final OutputWriter writer;

    public sol_ma_wa2(InputReader reader, OutputWriter writer) {
        this.reader = reader;
        this.writer = writer;
    }

    public static void main(String[] args) {
        InputReader reader = new InputReader(System.in);
        OutputWriter writer = new OutputWriter(System.out);
        new sol_ma_wa2(reader, writer).run();
        writer.writer.flush();
    }

    final int MOD = 42424243;

    class Pair {
        int x, y;
        Pair(int x, int y) {
            this.x = x;
            this.y = y;
        }
    }

    class PrefixFenwick {
        int[] F;
        int[] cnt;
        PrefixFenwick(int n) {
            F = new int[n];
            cnt = new int[n];
            Arrays.fill(cnt, 1);
        }
        void upd(int x, int y, int c) {
            for (; x < F.length; x |= x + 1) {
                if (F[x] < y) {
                    F[x] = y;
                    cnt[x] = c;
                } else if (F[x] == y) {
                    cnt[x] += c;
                    if (cnt[x] >= MOD)
                        cnt[x] -= MOD;
                } else {
                    break;
                }
            }
        }
        Pair get(int x) {
            int ans = 0, ansc = 1;
            for (; x >= 0; x &= x + 1, --x) {
                if (ans < F[x]) {
                    ans = F[x];
                    ansc = cnt[x];
                } else if (ans == F[x]) {
                    ansc += cnt[x];
                    if (ansc >= MOD)
                        ansc -= MOD;
                }
            }
            return new Pair(ans, ansc);
        }
    }
    class SuffixFenwick {
        int[] F;
        int[] cnt;
        SuffixFenwick(int n) {
            F = new int[n];
            cnt = new int[n];
            Arrays.fill(cnt, 1);
        }
        void upd(int x, int y, int c) {
            for (; x >= 0; x &= x + 1, --x) {
                if (F[x] < y) {
                    F[x] = y;
                    cnt[x] = c;
                } else if (F[x] == y) {
                    cnt[x] += c;
                    if (cnt[x] >= MOD)
                        cnt[x] -= MOD;
                } else {
                    break;
                }
            }
        }
        Pair get(int x) {
            int ans = 0, ansc = 1;
            for (; x < F.length; x |= x + 1) {
                if (ans < F[x]) {
                    ans = F[x];
                    ansc = cnt[x];
                } else if (ans == F[x]) {
                    ansc += cnt[x];
                    if (ansc >= MOD)
                        ansc -= MOD;
                }
            }
            return new Pair(ans, ansc);
        }
    }
    Query[] firstByX;

    class Query {
        int x, y, id;
        Query nxt;
        int leftBelow, rightAbove;
        int ans;

        Query(int x, int y, int id) {
            this.x = x;
            this.y = y;
            this.id = id;
            nxt = firstByX[x];
            firstByX[x] = this;
            leftBelow = rightAbove = -1;
        }

    }

    int LowerBound(int[] Hs, int y) {
        int l = -1, r = Hs.length;
        while (r - l > 1) {
            int q = (l + r) / 2;
            if (Hs[q] >= y)
                r = q;
            else
                l = q;
        }
        return r;
    }

    int UpperBound(int[] Hs, int y) {
        int l = -1, r = Hs.length;
        while (r - l > 1) {
            int q = (l + r) / 2;
            if (Hs[q] > y)
                r = q;
            else
                l = q;
        }
        return r;
    }

    public void run() {
        int n = reader.nextInt();
        int m = reader.nextInt();
        int[] H = new int[n];
        for (int i = 0; i < n; i++) {
            H[i] = reader.nextInt();
        }

        int[] Hs = H.clone();
        Arrays.sort(Hs);
        int pt = 1;
        for (int i = 1; i < Hs.length; i++) {
            if (Hs[i] != Hs[pt - 1])
                Hs[pt++] = Hs[i];
        }
        Hs = Arrays.copyOf(Hs, pt);

        Query[] Q = new Query[m];
        firstByX = new Query[n];
        for (int i = 0; i < m; i++) {
            int x = reader.nextInt() - 1;
            int y = reader.nextInt();
            Q[i] = new Query(x, y, i);
        }

        Pair[] endingHere = new Pair[n], startingHere = new Pair[n];

        PrefixFenwick PF = new PrefixFenwick(Hs.length);
        for (int i = 0; i < n; i++) {
            for (Query q = firstByX[i]; q != null; q = q.nxt) {
                Pair p = PF.get(LowerBound(Hs, q.y) - 1);
                q.leftBelow = p.x;
            }
            int pos = LowerBound(Hs, H[i]);
            Pair val = PF.get(pos - 1);
            endingHere[i] = val;
            PF.upd(pos, val.x + 1, val.y);
        }

        Pair total = PF.get(Hs.length - 1);
        int maxLis = total.x, cntMaxLis = total.y;

        SuffixFenwick SF = new SuffixFenwick(Hs.length);
        for (int i = n - 1; i >= 0; i--) {
            for (Query q = firstByX[i]; q != null; q = q.nxt) {
                Pair p = SF.get(UpperBound(Hs, q.y));
                q.rightAbove = p.x;
            }
            int pos = LowerBound(Hs, H[i]);
            Pair val = SF.get(pos + 1);
            startingHere[i] = val;
            SF.upd(pos, val.x + 1, val.y);
        }

        for (Query q : Q) {
            if (q.leftBelow + q.rightAbove + 1 >= maxLis)
                q.ans = q.leftBelow + q.rightAbove + 1;
            else {
                int val = endingHere[q.x].x + startingHere[q.x].x + 1;
                int cnt = (int)((((long)endingHere[q.x].y) * startingHere[q.x].y) % MOD);
                if (val != maxLis || cnt != cntMaxLis)
                    q.ans = maxLis;
                else
                    q.ans = maxLis - 1;
            }
        }

        for (Query q : Q) {
            writer.printf("%d\n", q.ans);
        }
    }


    static class InputReader {
        public BufferedReader reader;
        public StringTokenizer tokenizer;

        public InputReader(InputStream stream) {
            reader = new BufferedReader(new InputStreamReader(stream), 32768);
            tokenizer = null;
        }

        public String next() {
            while (tokenizer == null || !tokenizer.hasMoreTokens()) {
                try {
                    tokenizer = new StringTokenizer(reader.readLine());
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }
            return tokenizer.nextToken();
        }

        public int nextInt() {
            return Integer.parseInt(next());
        }

        public double nextDouble() {
            return Double.parseDouble(next());
        }

        public long nextLong() {
            return Long.parseLong(next());
        }
    }

    static class OutputWriter {
        public PrintWriter writer;

        OutputWriter(OutputStream stream) {
            writer = new PrintWriter(stream);
        }

        public void printf(String format, Object... args) {
            writer.print(String.format(Locale.ENGLISH, format, args));
        }
    }
}
