#include <iostream>
#include <fstream>
#include <set>
#include <map>
#include <string>
#include <vector>
#include <bitset>
#include <algorithm>
#include <cstring>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <queue>


typedef long long ll;
typedef long double ld;

using namespace std;
const int MAXN = 510000;
const int MOD = 1000000007;

pair<int, int> rmq2[MAXN * 4];
int n, m;
int fl[MAXN];
int a[MAXN];
pair<int, int> gol[MAXN];
pair<int, int> gor[MAXN];

pair<int, int> merge(pair<int, int> x, pair<int, int> y) {
	if (x.first < y.first)
		swap(x, y);
	if (x.first == y.first) {
		x.second += y.second;
		if (x.second >= MOD)
			x.second -= MOD;
	}
	return x;
}

void build(int v, int tl, int tr) {
	rmq2[v] = make_pair(0, 0);
	if (tl + 1 == tr)
		return;
	int m = (tl + tr) >> 1;
	build(v * 2 + 1, tl, m);
	build(v * 2 + 2, m, tr);
}

void upd(int v, int tl, int tr, int x, pair<int, int> y) {
	rmq2[v] = merge(rmq2[v], y);
	if (tl + 1 == tr)
		return;
	int m = (tl + tr) >> 1;
	if (x < m)
		upd(v * 2 + 1, tl, m, x, y);
	else
		upd(v * 2 + 2, m, tr, x, y);
}

pair<int, int> get(int v, int tl, int tr, int l, int r) {
	if (r <= tl || tr <= l)
		return make_pair(0, 0);
	if (l <= tl && tr <= r)
		return rmq2[v];
	int m = (tl + tr) >> 1;
	return merge(get(v * 2 + 1, tl, m, l, r), get(v * 2 + 2, m, tr, l, r));
}



struct rmq1d {
	vector<int> rmq;
	vector<int> vv;
	void add(int x) {
		vv.push_back(x);
	}
	void build(int v, int tl, int tr) {
		if (tl + 1 == tr) {
			rmq[v] = vv[tl];
			return;
		}
		int m = (tl + tr) >> 1;
		build(v * 2 + 1, tl, m);
		build(v * 2 + 2, m, tr);
		rmq[v] = max(rmq[v * 2 + 1], rmq[v * 2 + 2]);
	}
	void init() {
		rmq.resize(vv.size() * 4);
		build(0, 0, (int)vv.size());
	}
	int get(int v, int tl,int tr, int l, int r) {
		if (r <= tl || tr <= l)
			return 0;
		if (l <= tl && tr <= r)
			return rmq[v];
		int m = (tl + tr) >> 1;
		return max(get(v * 2 + 1, tl, m, l, r), get(v * 2 + 2, m, tr, l, r));
	}
	int get(int l, int r) {
		return get(0, 0, (int)vv.size(), l, r);
	}
};


struct rmq2d {
	vector<rmq1d> rmq;
	vector<vector<pair<int, int> > > vv;
	int n;
	void init(int N) {
		n = N;
		vv.resize(4 * n);
		rmq.resize(4 * n);
	}
	void add(int v, int tl, int tr, int x, int y, int d) {
		vv[v].push_back(make_pair(y, d));
		if (tl + 1 == tr)
			return;
		int m = (tl + tr) >> 1;
		if (x < m)
			add(v * 2 + 1, tl, m, x, y, d);
		else
			add(v * 2 + 2, m, tr, x, y, d);
	}
	void add(int x, int y, int d) {
		add(0, 0, n, x, y, d);
	}
	void build(int v, int tl, int tr) {
		sort(vv[v].begin(), vv[v].end());
		for (int i = 0; i < (int)vv[v].size(); ++i)
			rmq[v].add(vv[v][i].second);
		rmq[v].init();
		if (tl + 1 == tr)
			return;
		int m = (tl + tr) >> 1;
		build(v * 2 + 1, tl, m);
		build(v * 2 + 2, m, tr);
	}
	void init2() {
		build(0, 0, n);
	}
	int get(int v, int tl, int tr, int l1, int r1, int l2, int r2) {
		if (r1 <= tl || tr <= l1)
			return 0;
		if (l1 <= tl && tr <= r1) {
			int l = lower_bound(vv[v].begin(), vv[v].end(), make_pair(l2, 0)) - vv[v].begin();
			int r = lower_bound(vv[v].begin(), vv[v].end(), make_pair(r2, 0)) - vv[v].begin();
			return rmq[v].get(l, r);
		}
		int m = (tl + tr) >> 1;
		return max(get(v * 2 + 1, tl, m, l1, r1, l2, r2), get(v * 2 + 2, m, tr, l1, r1, l2, r2));
	}
};






vector<int> vv;

int main() {
	scanf("%d%d", &n, &m);
	for (int i = 0; i < n; ++i)
		scanf("%d", a + i), vv.push_back(a[i]);
	sort(vv.begin(), vv.end());
	vv.resize(unique(vv.begin(), vv.end()) - vv.begin());
	for (int i = 0; i < n; ++i)
		a[i] = lower_bound(vv.begin(), vv.end(), a[i]) - vv.begin();

	for (int i = 0; i < n; ++i) {
		gol[i] = get(0, 0, n, 0, a[i]);
		++gol[i].first;
		if (gol[i].first == 1)
			gol[i].second = 1;
		upd(0, 0, n, a[i], gol[i]);
	}
	pair<int, int> mx = rmq2[0];
	build(0, 0, n);
	for (int i = n - 1; i >= 0; --i) {
		gor[i] = get(0, 0, n, a[i] + 1, n);
		++gor[i].first;
		if (gor[i].first == 1)
			gor[i].second = 1;
		upd(0, 0, n, a[i], gor[i]);
	}

	for (int i = 0; i < n; ++i)
		if (mx.first == gol[i].first + gor[i].first - 1 && ((ll)gol[i].second * gor[i].second) % MOD == mx.second)
			fl[i] = 1;

	rmq2d rr1;
	rr1.init(n);
	for (int i = 0; i < n; ++i)
		rr1.add(i, a[i], gol[i].first);
	rr1.init2();
	rmq2d rr2;
	rr2.init(n);
	for (int i = 0; i < n; ++i)
		rr2.add(i, a[i], gor[i].first);
	rr2.init2();

	for (int i = 0; i < m; ++i) {
		int x, y;
		scanf("%d%d", &x, &y);
		--x;
		int ans = mx.first - fl[x];
		int y2 = lower_bound(vv.begin(), vv.end(), y) - vv.begin();
		int y3 = upper_bound(vv.begin(), vv.end(), y) - vv.begin();
		ans = max(ans, rr1.get(0, 0, n, 0, x, 0, y2) + 1 + rr2.get(0, 0, n, x + 1, n, y3, n));
		printf("%d\n", ans);
	}

	return 0;
}


