#include "testlib.h"

#include <iostream>
#include <set>
#include <vector>

int const MAX_VALUE = 1000 * 1000 * 1000;

int next_value(std::set<int> &values) {
    int curr;
    do {
        curr = rnd.next(1, MAX_VALUE);
    } while (values.find(curr) != values.end());
    values.insert(curr);
    return curr;
}

void print_test(std::vector<std::vector<int> > const &matrix) {
    int n = (int)matrix.size(), m = (int)matrix[0].size();
    std::vector<std::pair<int, int> > vals(n * m);
    std::set<int> distinct;
    
    for (int i = 0; i != n; ++i) {
        for (int j = 0; j != m; ++j) {
            vals[i * m + j] = std::make_pair(matrix[i][j], i * m + j);
            distinct.insert(matrix[i][j]);
        }
    }
    
    sort(vals.begin(), vals.end());
    std::set<int> gen;
    std::vector<int> new_vals(distinct.size());
    for (int i = 0; i != new_vals.size(); ++i) {
        new_vals[i] = next_value(gen);
    }
    
    std::sort(new_vals.begin(), new_vals.end());
    
    std::vector<std::vector<int> > res(n, std::vector<int>(m));
    std::vector<int>::const_iterator last = new_vals.begin();
    
    for (int i = 0; i != n * m; ++i) {
        if (i != 0 && vals[i].first != vals[i - 1].first) {
            ++last;
        }
        res[vals[i].second / m][vals[i].second % m] = *last;
    }
    
    std::cout << n << ' ' << m << '\n';
    for (int i = 0; i != n; ++i) {
        for (int j = 0; j != m; ++j) {
            std::cout << res[i][j];
            if (j != m - 1) {
                std::cout << ' ';
            }
        }
        std::cout << '\n';
    }
}

int main(int argc, char **argv) {
    registerGen(argc, argv, 1);
    
    ensure(argc >= 2);
    
    int n = atoi(argv[1]);
    int m = atoi(argv[2]);
    
    std::vector<std::vector<int> > matrix(std::max(n, m) * 10, std::vector<int>(std::max(n, m) * 10));
    
    for (int i = 0; i != n; ++i) {
        matrix[i][0] = -1;
        matrix[i][m - 1] = -1;
    }
    for (int j = 0; j != m; ++j) {
        matrix[0][j] = -1;
        matrix[n - 1][j] = -1;
    }
    
    int x = (int)matrix.size() / 2, y = (int)matrix.size() / 2, d = 0, val = 1;
    int const dx[] = {-1, 0, 1, 0};
    int const dy[] = {0, -1, 0, 1};
    
    while (matrix[x][y] == 0) {
        matrix[x][y] = val++;
        x += dx[d];
        y += dy[d];
        if (matrix[x + dx[(d + 1) % 4]][y + dy[(d + 1) % 4]] == 0) {
            d = (d + 1) % 4;
        }
    }
    
    int xs = (int)matrix.size() / 2 - n / 3;
    int ys = (int)matrix.size() / 2 - m / 3 * 2;
    
    std::vector<std::vector<int> > res(n, std::vector<int>(m));
    for (int i = 0; i != n; ++i) {
        for (int j = 0; j != m; ++j) {
            res[i][j] = matrix[xs + i][ys + j];
        }
    }
    
    print_test(res);
    
    return 0;
}