#include <cstdio>
#include <memory.h>
#include <algorithm>
#include <cassert>
#include <unordered_set>
using namespace std;

const int N = 500500;

int par[N];
int rnk[N];

int first[N];
int nxt[2 * N];
int last[N];
int deg[N];
int A[2 * N], B[2 * N];

int pop_first(int x) {
    assert(first[x] != -1);
    int e = first[x];
    if (nxt[e] == -1)
        first[x] = last[x] = -1;
    else
        first[x] = nxt[e];
    return e;
}

void push(int x, int e) {
    if (first[x] == -1) {
        first[x] = last[x] = e;
        nxt[e] = -1;
    } else {
        nxt[e] = first[x];
        first[x] = e;
    }
}

void absorb(int x, int y) {
    if (first[x] == -1) {
        first[x] = first[y];
        last[x] = last[y];
    } else {
        nxt[last[x]] = first[y];
        last[x] = last[y];
    }
    first[y] = last[y] = -1;
}

int get(int x) {
    return (x == par[x]) ? x : (par[x] = get(par[x]));
}

void merge(int a, int b) {
    a = get(a);
    b = get(b);
    if (a == b)
        return;
    if (rnk[a] > rnk[b])
        swap(a, b);
    if (rnk[a] == rnk[b])
        rnk[b]++;
    absorb(b, a); 
    par[a] = b;
}

int lpt = 0, rpt = 0;
int Q[N];
bool used[N];
vector<int> E[N];
vector<int> E2[N];


int main() {
    int n;
    scanf("%d", &n);
    for (int i = 0; i < n - 1; i++) {
        int a, b;
        scanf("%d %d", &a, &b);
        deg[a]++, deg[b]++;
        E[a].push_back(b);
        E[b].push_back(a);
    }
    memset(first + 1, -1, sizeof(int) * n);
    memset(last + 1, -1, sizeof(int) * n);
    for (int i = 1; i <= n; i++)
        par[i] = i;
    memset(nxt, -1, sizeof(int) * (2 * n - 2));
    for (int i = 0; i < n - 1; i++) {
        int a, b;
        scanf("%d %d", &a, &b);
        A[2 * i] = a, B[2 * i] = b;
        A[2 * i + 1] = b, B[2 * i + 1] = a;
        push(a, 2 * i);
        push(b, 2 * i + 1);
        E2[a].push_back(b);
        E2[b].push_back(a);
    }
    for (int i = 1; i <= n; i++) {
        if (deg[i] == 1)
            Q[rpt++] = i;
    }
    vector<tuple<int, int, int, int>> ans;
    while (lpt != rpt) {
        int x = Q[lpt++];
        used[x] = true;
        int y = -1;
        for (int v : E[x]) {
            if (used[v])
                continue;
            assert(y == -1);
            y = v;
        }
        if (y == -1) {
            assert(deg[x] == 0);
            continue;
        }
        assert(deg[x] == 1);
        deg[x]--;
        deg[y]--;
        if (deg[y] == 1)
            Q[rpt++] = y;
        if (find(E2[x].begin(), E2[x].end(), y) == E2[x].end()) {
            int a = x, b = y, c, d;
            int e;
            x = get(x);
            while (true) {
                e = pop_first(x);
                y = B[e];
                if (get(y) != get(x))
                    break;
            }
            c = A[e], d = B[e];
            ans.emplace_back(a, b, c, d);
        }
        merge(x, y);
    }
    printf("%d\n", (int)ans.size());
    for (auto oper : ans) {
        int a, b, c, d;
        tie(a, b, c, d) = oper;
        printf("%d %d %d %d\n", a, b, c, d);
    }
    return 0;
}
